package Class::Maker::Basic::Types;

{
	package Class::Maker::Basic::Types::int;

	sub get
	{
	}

	sub set
	{
	}
}

{
	package Class::Maker::Basic::Types::string;

	sub get
	{
	}

	sub set
	{
	}
}

1;
