
use Class::Maker::Examples::Array;
use Class::Maker::Examples::Auth;
use Class::Maker::Examples::Basket;
use Class::Maker::Examples::Bouncer;
use Class::Maker::Examples::Customer;
use Class::Maker::Examples::Employee;
use Class::Maker::Examples::Expirable;
use Class::Maker::Examples::Human;
use Class::Maker::Examples::Lockable;
use Class::Maker::Examples::Obsessor;
use Class::Maker::Examples::Shop;
use Class::Maker::Examples::Trustee;
use Class::Maker::Examples::User;

1;
